{ Exemplo de Relgio Analgico feito em Delphi.

  Autor....: Daniel Pereira Guimares
  E-mail...: tecnobyte@ulbrajp.com.br
  Home-Page: www.ulbrajp.com.br/~tecnobyte
  Data.....: 19-Julho-2002.

  IMPORTANTE!

  Este software poder ser re-distribudo sem nenhum custo. Peo
  apenas que mantenha meus crditos como autor.

  Ficarei muito grato se receber suas crticas, sugestes e
  comentrios.
}

unit uRelogio;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, StdCtrls;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    lblNome: TLabel;
    lblEmail: TLabel;
    lblHomePage: TLabel;
    procedure Timer1Timer(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    Hora: TDateTime;
    CentroX, CentroY: integer;
    procedure DesenharMostrador;
    procedure DesenharPonteiros(const Hora: TDateTime; const Apagar: boolean);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

const
  { Comprimento (em pixels) do raio da circunferncia e
    dos ponteiros do relgio }
  CompRaio = 100;
  CompSegundo = 90;
  CompMinuto = 80;
  CompHora = 60;

  { Espessura (em pixels) da circunferncia e dos ponteiros do relgio }
  EspCirc = 2;
  EspMarca = 1;
  EspSegundo = 1;
  EspMinuto = 3;
  EspHora = 5;

  { Cores }
  CorFundo = clWhite;
  CorCirc = clBlack;
  CorMarca1 = clRed;
  CorMarca2 = clYellow;
  CorSegundo = clRed;
  CorMinuto = clBlue;
  CorHora = clNavy;

procedure TForm1.DesenharMostrador;
var
  Angulo, X, Y: Extended;
  P: TPoint;
  I, J: integer;
begin
  with Canvas do begin
    { Define cor do fundo }
    Brush.Color := CorFundo;

    { Define cor e espessura da caneta (linha) }
    Pen.Color := CorCirc;
    Pen.Width := EspCirc;

    { Desenha o crculo que define o mostrador }
    Ellipse(CentroX - CompRaio, CentroY - CompRaio,
      CentroX + CompRaio, CentroY + CompRaio);

    { Desenha as doze marcas das horas }
    Pen.Width := EspMarca;

    for I := 1 to 12 do
    begin
      { Se for 3, 6, 9 ou 12 horas... }
      if I mod 3 = 0 then
      begin
        J := 3;
        Brush.Color := CorMarca1;
      end else begin
        J := 2;
        Brush.Color := CorMarca2;
      end;

      { Calcula a posio da marca }
      Angulo := DegToRad(90 - 360 * I / 12);
      X := Cos(Angulo);
      Y := Sin(Angulo);
      P.x := CentroX + Trunc(CompRaio * X);
      P.y := CentroY - Trunc(CompRaio * Y);

      { Desenha  a marca na posio calculada }
      Ellipse(P.x - J, P.y - J, P.x + J, P.y + J);
    end;
  end;
end;

procedure TForm1.DesenharPonteiros(const Hora: TDateTime; const Apagar: boolean);
var
  H, M, S, MSeg: Word;
  A, Angulo, X, Y: Extended;
  P: TPoint;
begin
  { Decodifica a hora }
  DecodeTime(Hora, H, M, S, MSeg);

  with Canvas do begin
    { Desenha o ponteiro das horas }
    Pen.Width := EspHora;
    if Apagar then
      Pen.Color := CorFundo
    else
      Pen.Color := CorHora;

    A := H mod 12 + M / 60 + S / 3600;
    Angulo := DegToRad(90 - 360 * A / 12);
    X := Cos(Angulo);
    Y := Sin(Angulo);
    P.x := CentroX + Trunc(X * CompHora);
    P.y := CentroY - Trunc(Y * CompHora);

    MoveTo(CentroX, CentroY);
    LineTo(P.x, P.y);

    { Desnha o ponteiro dos minutos }
    Pen.Width := EspMinuto;
    if Apagar then
      Pen.Color := CorFundo
    else
      Pen.Color := CorMinuto;

    A := M + S / 60;
    Angulo := DegToRad(90 - 360 * A / 60);
    X := Cos(Angulo);
    Y := Sin(Angulo);
    P.x := CentroX + Trunc(X * CompMinuto);
    P.y := CentroY - Trunc(Y * CompMinuto);

    MoveTo(CentroX, CentroY);
    LineTo(P.x, P.y);

    { Desnha o ponteiro dos segundos }
    Pen.Width := EspSegundo;
    if Apagar then
      Pen.Color := CorFundo
    else
      Pen.Color := CorSegundo;

    A := S;
    Angulo := DegToRad(90 - 360 * A / 60);
    X := Cos(Angulo);
    Y := Sin(Angulo);
    P.x := CentroX + Trunc(X * CompSegundo);
    P.y := CentroY - Trunc(Y * CompSegundo);

    MoveTo(CentroX, CentroY);
    LineTo(P.x, P.y);
  end;

end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  { Apaga os ponteiros atuais. }
  DesenharPonteiros(Hora, true);

  { Desenha novamente os ponteiros }
  Hora := Time;
  DesenharPonteiros(Hora, false);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Canvas.Pen.Mode := pmNot;
  Canvas.MoveTo(0,0);
  Canvas.LineTo(100, 100);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  Canvas.Brush.Color := clBlue;
  Canvas.Pen.Width := 3;
  Canvas.Ellipse(0,0,100,100);
end;

procedure TForm1.FormCreate(Sender: TObject);
const
  Margem = 10;
  Espaco = 5;
begin
  CentroX := ClientWidth div 2;
  CentroY := ClientHeight div 2;

  lblNome.Left := (ClientWidth - lblNome.Width) div 2;
  lblEmail.Left := (ClientWidth - lblEmail.Width) div 2;
  lblHomePage.Left := (ClientWidth - lblHomePage.Width) div 2;

  lblHomePage.Top := ClientHeight - Margem - lblHomePage.Height;
  lblEmail.Top := lblHomePage.Top - Espaco - lblEmail.Height;
  lblNome.Top := lblEmail.Top - Espaco - lblNome.Height;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  DesenharMostrador;
  Hora := Time;
  DesenharPonteiros(Hora, false);
end;

end.
